import {
  ApiQueryBuilderRequest,
  substrReplace
} from "./chunk-MKQ3HMMA.js";
import {
  InvalidQuery
} from "./chunk-XIRHVLVQ.js";

// src/filters/filters_exact.ts
import collect from "collect.js";
var FiltersExact = class {
  constructor(addRelationConstraint = true) {
    this.addRelationConstraint = addRelationConstraint;
  }
  relationConstraints = [];
  handle(query, value, property) {
    if (this.addRelationConstraint && this.isRelationProperty(query, property)) {
      this.withRelationConstraint(query, value, property);
      return;
    }
    if (Array.isArray(value)) {
      void query.whereIn(this.qualifyColumn(query, property), value);
      return;
    }
    if (value === null) {
      void query.whereNull(this.qualifyColumn(query, property));
      return;
    }
    void query.where(this.qualifyColumn(query, property), "=", value);
  }
  qualifyColumn(query, column) {
    if (column.includes(".")) {
      return column;
    }
    return `${query.model.table}.${query.model.$getColumn(column)?.columnName}`;
  }
  isRelationProperty(query, property) {
    if (!property.includes(".")) {
      return false;
    }
    if (this.relationConstraints.includes(property)) {
      return false;
    }
    const firstRelationShip = property.split(".")[0];
    return query.model.$relationsDefinitions.has(firstRelationShip);
  }
  withRelationConstraint(query, value, property) {
    const [relation, relationProperty] = collect(property.split(".")).pipe((parts) => {
      const rProperty = parts.pop();
      const rPath = parts.implode(".");
      return [rPath, rProperty];
    });
    void query.whereHas(relation, (subQuery) => {
      const columnName = this.qualifyColumn(subQuery, relationProperty);
      this.relationConstraints.push(columnName);
      this.handle(subQuery, value, columnName);
    });
  }
};

// src/filters/filters_partial.ts
var FiltersPartial = class _FiltersPartial extends FiltersExact {
  handle(query, value, property) {
    if (this.addRelationConstraint && this.isRelationProperty(query, property)) {
      this.withRelationConstraint(query, value, property);
      return;
    }
    if (Array.isArray(value)) {
      if (value.filter((item) => item.toString().length > 0).length === 0) {
        return;
      }
      void query.where((subQuery) => {
        for (const partialValue of value.filter((item) => item.toString().length > 0)) {
          const [innerSql, innerBindings] = this.getWhereRawParameters(
            partialValue,
            this.parsePropertyToColumn(subQuery, property)
          );
          void subQuery.orWhereRaw(innerSql, innerBindings);
        }
      });
      return;
    }
    const [sql, bindings] = this.getWhereRawParameters(value, this.parsePropertyToColumn(query, property));
    void query.whereRaw(sql, bindings);
  }
  parsePropertyToColumn(query, property) {
    if (property.includes(".")) {
      return property;
    }
    return query.model.$getColumn(property)?.columnName ?? property;
  }
  getDatabaseDialect(query) {
    return query.client.dialect.name;
  }
  getWhereRawParameters(value, property) {
    const resolvedValue = `${value}`.toLowerCase();
    return [`LOWER(??) LIKE ?`, [property, `%${_FiltersPartial.escapeLike(resolvedValue)}%`]];
  }
  static escapeLike(value) {
    return value.replaceAll("\\", "\\\\").replaceAll("_", String.raw`\\_`).replaceAll("%", String.raw`\\%`);
  }
};

// src/filters/filters_begin_with_strict.ts
var FiltersBeginWithStrict = class _FiltersBeginWithStrict extends FiltersPartial {
  getWhereRawParameters(value, property) {
    return [`${property} LIKE ?`, [`${_FiltersBeginWithStrict.escapeLike(value.toString())}%`]];
  }
};

// src/filters/filters_callback.ts
var FiltersCallback = class {
  constructor(callback) {
    this.callback = callback;
  }
  handle(query, value, property) {
    this.callback(query, value, property);
  }
};

// src/filters/filters_ends_with_strict.ts
var FiltersEndsWithStrict = class _FiltersEndsWithStrict extends FiltersPartial {
  getWhereRawParameters(value, property) {
    return [`${property} LIKE ?`, [`%${_FiltersEndsWithStrict.escapeLike(value.toString())}`]];
  }
};

// src/enums/filter_operator.ts
var FilterOperator = {
  Dynamic: "",
  Equal: "=",
  LessThan: "<",
  GreaterThan: ">",
  LessThanOrEqual: "<=",
  GreaterThanOrEqual: ">=",
  NotEqual: "<>"
};

// src/filters/filters_operator.ts
var FiltersOperator = class extends FiltersExact {
  constructor($addRelationConstraint, $filterOperator, $boolean) {
    super($addRelationConstraint);
    this.$addRelationConstraint = $addRelationConstraint;
    this.$filterOperator = $filterOperator;
    this.$boolean = $boolean;
  }
  handle(query, value, property) {
    let filterOperator = this.$filterOperator;
    if (this.$addRelationConstraint && this.isRelationProperty(query, property)) {
      this.withRelationConstraint(query, value, property);
      return;
    }
    if (Array.isArray(value)) {
      const items = value;
      void query.where((subQuery) => {
        for (const item of items) {
          this.handle(subQuery, item, property);
        }
      });
      return;
    }
    if (filterOperator === FilterOperator.Dynamic) {
      filterOperator = this.getDynamicFilterOperator(value.toString());
      value = this.removeDynamicFilterOperatorFromValue(value.toString(), filterOperator);
    }
    if (this.$boolean === "and") {
      void query.where(this.qualifyColumn(query, property), filterOperator, value);
      return;
    }
    void query.orWhere(this.qualifyColumn(query, property), filterOperator, value);
  }
  getDynamicFilterOperator(value) {
    let filterOperator = FilterOperator.Equal;
    for (const filterOperatorCase of Object.values(FilterOperator)) {
      if (value.startsWith(filterOperatorCase) && filterOperatorCase !== FilterOperator.Dynamic) {
        filterOperator = filterOperatorCase;
      }
    }
    return filterOperator;
  }
  removeDynamicFilterOperatorFromValue(value, filterOperator) {
    if (value.includes(filterOperator)) {
      value = substrReplace(value, "", 0, filterOperator.length);
    }
    return value;
  }
};

// src/filters/filters_trashed.ts
var FiltersTrashed = class {
  handle(query, value, _property) {
    if (!("withTrashed" in query && "onlyTrashed" in query)) {
      return;
    }
    const queryWithSoftDeletes = query;
    if (value === "with") {
      void queryWithSoftDeletes.withTrashed();
      return;
    }
    if (value === "only") {
      void queryWithSoftDeletes.onlyTrashed();
    }
  }
};

// src/allowed_filter.ts
import { Collection } from "collect.js";
var AllowedFilter = class _AllowedFilter {
  static setFilterArrayValueDelimiter(delimiter) {
    if (delimiter) {
      ApiQueryBuilderRequest.setFilterArrayValueDelimiter(delimiter);
    }
  }
  static exact(name, internalName, addRelationConstraint = true, arrayValueDelimiter) {
    this.setFilterArrayValueDelimiter(arrayValueDelimiter);
    return new _AllowedFilter(name, new FiltersExact(addRelationConstraint), internalName);
  }
  static partial(name, internalName, addRelationConstraint = true, arrayValueDelimiter) {
    this.setFilterArrayValueDelimiter(arrayValueDelimiter);
    return new _AllowedFilter(name, new FiltersPartial(addRelationConstraint), internalName);
  }
  static beginsWithStrict(name, internalName, addRelationConstraint = true, arrayValueDelimiter) {
    this.setFilterArrayValueDelimiter(arrayValueDelimiter);
    return new _AllowedFilter(name, new FiltersBeginWithStrict(addRelationConstraint), internalName);
  }
  static endsWithStrict(name, internalName, addRelationConstraint = true, arrayValueDelimiter) {
    this.setFilterArrayValueDelimiter(arrayValueDelimiter);
    return new _AllowedFilter(name, new FiltersEndsWithStrict(addRelationConstraint), internalName);
  }
  static callback(name, callback, internalName, arrayValueDelimiter) {
    this.setFilterArrayValueDelimiter(arrayValueDelimiter);
    return new _AllowedFilter(name, new FiltersCallback(callback), internalName);
  }
  static trashed(name = "trashed", internalName) {
    return new _AllowedFilter(name, new FiltersTrashed(), internalName);
  }
  static custom(name, filterClass, internalName, arrayValueDelimiter) {
    this.setFilterArrayValueDelimiter(arrayValueDelimiter);
    return new _AllowedFilter(name, filterClass, internalName);
  }
  static operator(name, filterOperator, boolean = "and", internalName, addRelationConstraint = true, arrayValueDelimiter) {
    this.setFilterArrayValueDelimiter(arrayValueDelimiter);
    return new _AllowedFilter(
      name,
      new FiltersOperator(addRelationConstraint, filterOperator, boolean),
      internalName
    );
  }
  name;
  filterClass;
  internalName;
  ignored;
  default;
  $hasDefault = false;
  nullable = false;
  constructor(name, filterClass, internalName) {
    this.name = name;
    this.filterClass = filterClass;
    this.ignored = new Collection();
    this.internalName = internalName ?? this.name;
  }
  filter(query, value) {
    const valueToFilter = this.resolveValueForFiltering(value);
    if (!this.nullable && valueToFilter === null) {
      return;
    }
    this.filterClass.handle(query, valueToFilter, this.internalName);
  }
  getFilterClass() {
    return this.filterClass;
  }
  getName() {
    return this.name;
  }
  isForFilter(filterName) {
    return this.name === filterName;
  }
  ignore(...values) {
    this.ignored = this.ignored.merge(values).flatten();
    return this;
  }
  getIgnored() {
    return this.ignored.toArray();
  }
  getInternalName() {
    return this.internalName;
  }
  setDefault(defaultValue) {
    this.default = defaultValue;
    this.$hasDefault = true;
    if (defaultValue === null) {
      this.setNullable(true);
    }
    return this;
  }
  getDefault() {
    return this.default;
  }
  hasDefault() {
    return this.$hasDefault;
  }
  setNullable(nullable = true) {
    this.nullable = nullable;
    return this;
  }
  unsetDefault() {
    this.$hasDefault = false;
    this.default = void 0;
    return this;
  }
  resolveValueForFiltering(value) {
    if (Array.isArray(value)) {
      const remainingProperties = value.map((subValue) => this.resolveValueForFiltering(subValue));
      return remainingProperties.length === 0 ? null : remainingProperties;
    }
    return this.ignored.contains(value) ? null : value;
  }
};

// src/exceptions/invalid_filter_query.ts
import { ResponseStatus } from "@adonisjs/core/http";
var InvalidFilterQuery = class _InvalidFilterQuery extends InvalidQuery {
  constructor(unknownFilters, allowedFilters) {
    const notAllowed = unknownFilters.join(", ");
    const allowed = allowedFilters.join(", ");
    const message = `Requested filter(s) \`${notAllowed}\` are not allowed. Allowed filter(s) are \`${allowed}\``;
    super(message, { status: ResponseStatus.BadRequest });
    this.unknownFilters = unknownFilters;
    this.allowedFilters = allowedFilters;
  }
  static filtersNotAllowed(unknownFilters, allowedFilters) {
    return new _InvalidFilterQuery(unknownFilters, allowedFilters);
  }
};

export {
  FiltersExact,
  FiltersPartial,
  FiltersBeginWithStrict,
  FiltersCallback,
  FiltersEndsWithStrict,
  FilterOperator,
  FiltersOperator,
  FiltersTrashed,
  AllowedFilter,
  InvalidFilterQuery
};
//# sourceMappingURL=chunk-54HNQTDY.js.map